@@ -42,6 +42,7 @@ import getpass
 import email
 import poplib
 import imaplib
+import re
 
 try:
     # do we have a recent pykerberos?
@@ -410,11 +411,14 @@ class RetrieverSkeleton(ConfigurableBase):
         else:
             # Explicitly set to None in case it was previously set
             socket.setdefaulttimeout(None)
-        self.oldmail_filename = os.path.join(
-            self.conf['getmaildir'],
-            ('oldmail-%(server)s-%(port)i-%(username)s'
-             % self.conf).replace('/', '-').replace(':', '-')
+        # strip problematic characters from oldmail filename.  Mostly for
+        # non-Unix systems; only / is illegal in a Unix path component
+        oldmail_filename = re.sub(
+            r'[/\:;<>|]+', '-',
+            'oldmail-%(server)s-%(port)i-%(username)s' % self.conf
         )
+        self.oldmail_filename = os.path.join(self.conf['getmaildir'], 
+                                             oldmail_filename)
         self._read_oldmailfile()
         self.received_from = '%s (%s)' % (
             self.conf['server'],
@@ -785,7 +789,7 @@ class IMAPRetrieverBase(RetrieverSkeleton):
         self.log.debug('selecting mailbox "%s"' % mailbox + os.linesep)
         try:
             response = self._parse_imapcmdresponse('SELECT', mailbox)
-            count = int(response[0])
+            count = int(response[-1]) # use *last* EXISTS returned
             uidvalidity = self.conn.response('UIDVALIDITY')[1][0]
         except imaplib.IMAP4.error, o:
             raise getmailOperationError('IMAP error (%s)' % o)
@@ -860,7 +864,7 @@ class IMAPRetrieverBase(RetrieverSkeleton):
                            + os.linesep)
             try:
                 response = self._parse_imapuidcmdresponse('FETCH', uid, part)
-            except imaplib.IMAP4.error, o:
+            except (imaplib.IMAP4.error, getmailOperationError), o:
                 # server gave a negative/NO response, most likely.  Bad server,
                 # no doughnut.
                 raise getmailRetrievalError(
